//
// Copyright (C) 2000 Software Resources International SA
//
// This code is provided on an 'as is' basis.
// Software Resources International SA carries no responsibility for
// the application of this code in any way or form.
//

//==========================================================================
// FILE: endian_conv.h
// Converts word and long word from one endianity to another:
// Little-endian (Intel, DEC) <-> big-endian (Sun)
//==========================================================================

// you should include this file into "definitions.h"

#ifndef __ENDIAN_CONV_H__
#define __ENDIAN_CONV_H__

// This calss represents a nonhost-ordered word
class alien_endian_word_t 
{
private:
    word_t alien_word;

public:
    // Construct and assign of host-ordered word
    inline alien_endian_word_t(const word_t naitive_word = 0) 
    : alien_word(((naitive_word >> 8) & 0xff) | ( (naitive_word & 0xff) << 8)) {};

    // Assigning of host-ordered word
    inline alien_endian_word_t& operator= (const word_t naitive_word) { 
        alien_word  = (((naitive_word >> 8) & 0xff) | ((naitive_word & 0xff) << 8));
        return *this;
    };

    // Convert to the host byte order
    inline operator const word_t() const { 
        return ((alien_word >> 8)& 0xff) | ((alien_word & 0xff)<<8);
    };

    // Any other operations are not allowed
    // Convert to the host byte order and perform required actions
};

// This calss represents a nonhost-ordered double-word
class alien_endian_dword_t 
{
private:
    dword_t alien_dword;

public:

    // Construct and assign host-ordered dword
    inline alien_endian_dword_t(const dword_t native_dword = 0L)
    : alien_dword(((native_dword & 0x000000ff) << 24) | 
                  ((native_dword & 0x0000ff00) << 8 ) | 
                  ((native_dword & 0x00ff0000) >> 8 ) | 
                  ((native_dword & 0xff000000) >> 24)) 
    {};

    // Assign host-ordered word
    inline alien_endian_dword_t& operator= (const dword_t native_dword)
    { 
        alien_dword = (
            ((native_dword & 0x000000ff) << 24) | 
            ((native_dword & 0x0000ff00) << 8 ) | 
            ((native_dword & 0x00ff0000) >> 8 ) | 
            ((native_dword & 0xff000000) >> 24)
        );
        return *this;
    };

    // Convert to host byte order
    inline operator const dword_t() const { 
        return (
            ((alien_dword & 0x000000ff) << 24) | 
            ((alien_dword & 0x0000ff00) << 8 ) | 
            ((alien_dword & 0x00ff0000) >> 8 ) | 
            ((alien_dword & 0xff000000) >> 24)
        );
    };

    // Any other operations are not allowed
    // Convert to the host byte order and perform required actions
};

// This class represents a nonhost-ordered 3bytes-word
class alien_endian_3bytes_t 
{
private:
    unsigned char alien[3];

public:

    inline alien_endian_3bytes_t(const dword_t host_word = 0L)
    {
        alien[0] = (host_word >> 16) & 0xff;
        alien[1] = (host_word >> 8) & 0xff;
        alien[3] = (host_word) & 0xff;
    };

    // Assign host-ordered word
    inline alien_endian_3bytes_t& operator= (const dword_t host_word) {
        alien[0] = (host_word >> 16) & 0xff;
        alien[1] = (host_word >> 8) & 0xff;
        alien[3] = (host_word) & 0xff;
        return *this;
    };
    
    // Convert to the host byte order
    inline operator const dword_t() const { 
        return (dword_t)((((udword_t)alien[0]) << 16) |
                         (((udword_t)alien[1]) << 8) |
                         (((udword_t)alien[2])));
    }

    // Any other operations are not allowed
    // Convert to the host byte order and perform required actions
};

typedef  alien_endian_word_t	big_endian_word_t;
typedef  alien_endian_dword_t	big_endian_dword_t;
typedef  alien_endian_3bytes_t	big_endian_3bytes_t;

#endif
