//
// Copyright (C) 2000 Software Resources International SA
//
// This code is provided on an 'as is' basis.
// Software Resources International SA carries no responsibility for
// the application of this code in any way or form.
//

// Files-11 Structure level 2 

#ifndef __F11FILE_H__
#define __F11FILE_H__

#include "files11.h"
#include "f11window.h"

//  Forward Declarations
class f11_volume_acp;

//  File header chain
struct file_hdr {
    ODS_FH2	fhdr;		// file header block
    lbn_t	fhdr_lbn;

    file_hdr*   prev;		// prev file (header)
    file_hdr*   next;		// next file (header)
};

//  File structure
class f11_file {
public:
    file_hdr*		file_header;		// link to file header chain
    f11_volume_acp*	dev;			// backlink to device

    // File content window
    file_window*	p_content_window;	// file contents

    int			flag_content_modified;
    int			flag_header_modified;

private:
    
    // File virtual read/write
    // This function uses buffer in p_window_content and
    // modifies window_start_lbn and window_start_vbn
    int	    f11_read_write_file_blk(dword_t vbn, 
				        dword_t blkcount, int write);
public:

    int	    f11_read_file_blk(dword_t vbn, dword_t blkcount) {
		return f11_read_write_file_blk(vbn, blkcount, 0);
	    }

    int	    f11_write_file_blk(dword_t vbn, dword_t blkcount) {
		return f11_read_write_file_blk(vbn, blkcount, 1);
	    }

    dword_t get_rp_blk_count() const;
    int	    delete_file();
    int	    f11_sync();

    void    init_empty_dir() {
		((ODS_dir_record*)(this->p_content_window->p_content))->dir_w_size = -1;
		this->flag_content_modified = 1;
		this->get_fhdr()->set_file_length(F11_BLOCK_SIZE);
		this->flag_header_modified = 1;
		this->f11_sync();
	    }
    void    set_backlink(const ODS_FID& backlink) {
		this->get_fhdr()->fh2_w_backlink = backlink;
		this->flag_header_modified = 1;
		this->f11_sync();
	    }

public:
    f11_file() : p_content_window(NULL),
		 file_header(NULL), dev(NULL),
		 flag_content_modified(0),
		 flag_header_modified(0)
    {}

    ~f11_file() {
	if (p_content_window != NULL)
	delete p_content_window;
	p_content_window = NULL;
    }

    // Check if the opened file has the desired backlink
    virtual int	is_valid(const ODS_FID* back_fid) const {
	return 1;
    }

    ODS_FH2*	get_fhdr() const { 
	return &file_header->fhdr; 
    }
};

#endif 	// __F11FILE_H__
