//
// Copyright (C) 2000 Software Resources International SA
//
// This code is provided on an 'as is' basis.
// Software Resources International SA carries no responsibility for
// the application of this code in any way or form.
//

//  Files-11 Structure level 2 

#include <stdlib.h>
#include <string.h>

#include "definitions.h"
#include "f11asrt.h"
#include "devio.h"
#include "files11.h"
#include "f11liberr.h"
#include "f11file.h"
#include "f11vol.h"
#include "f11file_sys.h"
#include "f11file_seq.h"


// Binary mode getc()
int f11_file_sequential::f11_bin_getc()
{
    dword_t	blk, blk_pos, eof_blk;
    dword_t	count;

    ASSERT(p_content_window);

    //	EOF reached ?
    if (window_pos >= (int)get_fhdr()->get_file_length()) {
	dev->set_error(F11_EOF);
	return -1;
    }

    blk     = window_pos / F11_BLOCK_SIZE;
    blk_pos = window_pos % (F11_BLOCK_SIZE * this->p_content_window->block_count);

    //	We need to read new block(s)
    if (blk >= this->p_content_window->start_vbn + this->p_content_window->block_count) 
    {
	ASSERT(!get_fhdr()->is_file_contig());

        eof_blk = (get_fhdr()->get_file_length() + 
                   F11_BLOCK_SIZE - 1) / F11_BLOCK_SIZE;

	ASSERT(eof_blk > blk);

	count = ((eof_blk - blk) >= this->p_content_window->block_count) ? this->p_content_window->block_count : eof_blk - blk;
	if (f11_read_file_blk(blk, count) != (int)count) {
	    return -1;
	}
    }

    window_pos++;
    return (int)*((byte_t*)p_content_window->p_content + blk_pos);
}


// Binary mode putc()
int f11_file_sequential::f11_bin_putc(char c)
{
    dword_t	blk, blk_pos, eof_blk;
    dword_t	exvbn = 0;
    dword_t	count;
    int		sts;

    ASSERT(p_content_window);

    blk	    = window_pos / (F11_BLOCK_SIZE);
    blk_pos = window_pos % (F11_BLOCK_SIZE * this->p_content_window->block_count);

    if (blk >= get_fhdr()->get_hiblk())
    {
        // It's necessary to extend file
	while (f_size) {
	    sts = dev->acp_sub_extend(this,
			1,		//fib_w_exctl_fib_v_extend,
			0,		//fib_w_exctl_fib_v_alcon,
			0,		//fib_w_exctl_fib_v_alconb,
			0,		//fib_w_exctl_fib_v_filcon,
			0,		//fib_w_exctl_fib_v_aldef,
			f_size,		//fib_l_exsz,
			exvbn		//fib_l_exvbn
		    );
	    if (sts == SS_NORMAL) break;

	    if (sts == SS_ERROR) {
		f_size /= 2;		// Cannot extend file
		if (f_size == 0) return -1;
	    }
	}
    }

    if (blk >= this->p_content_window->start_vbn + this->p_content_window->block_count) 
    {
	// Turn window to the next block(s)
	ASSERT(!get_fhdr()->is_file_contig());

        eof_blk = (get_fhdr()->get_file_length() + 
                   F11_BLOCK_SIZE - 1) / F11_BLOCK_SIZE;

	count = ((get_fhdr()->get_hiblk() - blk) >= this->p_content_window->block_count) ? this->p_content_window->block_count : get_fhdr()->get_hiblk() - blk;
	if (f11_read_file_blk(blk, count) != (int)count) {
	    return -1;
	}
    }

    TRACE(("f11_bin_putc() : window_pos = %ld\n", window_pos));

    // putc 
    *((byte_t*)p_content_window->p_content + blk_pos) = c;
    window_pos++;
    flag_content_modified = 1;

    get_fhdr()->set_file_length(window_pos);
    flag_header_modified = 1;

    blk	    = window_pos / (F11_BLOCK_SIZE);
    blk_pos = window_pos % (F11_BLOCK_SIZE * this->p_content_window->block_count);

    if (blk >= this->p_content_window->start_vbn + this->p_content_window->block_count) 
    {
	f11_sync();

        // Try to save block 
        TRACE(("save block %ld to the disk\n", this->p_content_window->start_vbn));
        if (f11_write_file_blk(this->p_content_window->start_vbn, this->p_content_window->block_count) == -1) 
	{
            TRACE(("error: write block 0%d, blk_count 0%d",
	           this->p_content_window->start_vbn, this->p_content_window->block_count));
	    return -1;
        }
    }
    return 0;
}

//  text mode getc()
int f11_file_sequential::f11_text_getc()
{
    dword_t	blk, blk_pos, eof_blk;
    dword_t	j = 0;
    dword_t	count;

    ASSERT(p_content_window);

    // handle getc state 
    switch (text_mode_cur_state) 
    {
	case gs_eol1:
	    TRACE(("text_getc: (win = \\n, unix, vms = ). cur_state = gs_eol2. rem_bytes=%d\n", text_mode_rem_bytes));
	    text_mode_cur_state = gs_eol2;
#if defined(_WIN32)
	    return (int)'\n';
#endif
	case gs_eol2:
	    if (window_pos & 1) {
		window_pos++;
	    }
	    break;
	default:
	    break;
    }

    // EOF reached ?
    if (window_pos >= (int)get_fhdr()->get_file_length()) {
        if (text_mode_cur_state == gs_eol2) {
	    dev->set_error(F11_EOF);
	    return -1;
        }
        else {
            if (text_mode_rem_bytes) {
		TRACE(("INVALID TEXT FORMAT!!!\n"));
		dev->set_error(F11_INVALID_TEXT_FILE);
		return -1;
            }
	TRACE(("text_getc: eol reached (win = \\r, unix, vms = \\n). cur_state = gs_eol1. rem_bytes=%d\n", text_mode_rem_bytes));
	text_mode_cur_state = gs_eol1;

#if defined(unix) || defined(__VMS)
	return (int)'\n';
#elif defined(_WIN32) 
	return (int)'\r';
#endif	    
        }	    
	dev->set_error(F11_EOF);
        return -1;
    }

    blk     = window_pos / F11_BLOCK_SIZE;
    blk_pos = window_pos % (F11_BLOCK_SIZE * this->p_content_window->block_count);

    //  We need to read new block(s)
    if (blk >= this->p_content_window->start_vbn + 
				this->p_content_window->block_count) {
	
	ASSERT(!get_fhdr()->is_file_contig());
        
	eof_blk = (get_fhdr()->get_file_length() + F11_BLOCK_SIZE - 1) / F11_BLOCK_SIZE;
	
	ASSERT(eof_blk > blk);

	count = ((eof_blk - blk) >= this->p_content_window->block_count) 
				? this->p_content_window->block_count : eof_blk - blk;

	if (f11_read_file_blk(blk, count) != (int)count) {
	    dev->set_error(F11_ERROR_READING_FILE);
	    return -1;
	}
    }

    // New record ? 
    if (text_mode_cur_state == gs_eol2) {
        
	// Handle sequental record type,
        // format and FCR (FCR is subset of the sequental rec type) attribute  
        if (get_fhdr()->fh2_w_recattr.fat_b_rtype.fat_v_fileorg == ACP_FAT::fat_c_sequental)
	    j += 2;
        if (get_fhdr()->fh2_w_recattr.fat_b_rattrib.rattr.fat_m_fortrancc)
	    j += 1;

	word_t sz = *(ods_word_t*)((byte_t*)p_content_window->p_content + blk_pos);
        text_mode_rem_bytes = sz;
        text_mode_cur_state = gs_trans;

        TRACE(("text_getc: new record, cur_state = gs_eol2. rem_bytes=%d, sz = %d, j = %d\n", 
					text_mode_rem_bytes, sz, j));

	window_pos += j;

	blk_pos = window_pos % (F11_BLOCK_SIZE * this->p_content_window->block_count);
	blk     = window_pos / F11_BLOCK_SIZE;
	
        // Maybe read new block ? 
        if ((!blk_pos || !(blk_pos - j))) {
	    ASSERT(!get_fhdr()->is_file_contig());

	    eof_blk = (get_fhdr()->get_file_length() + F11_BLOCK_SIZE - 1) / F11_BLOCK_SIZE;

	    ASSERT(eof_blk > blk);

	    count = ((eof_blk - blk) >= this->p_content_window->block_count) 
					? this->p_content_window->block_count : eof_blk - blk;

	    if (f11_read_file_blk(blk, count) != (int)count) {
		dev->set_error(F11_ERROR_READING_FILE);
		return -1;
	    }
        }
    }

    // EOL reached ?
    if (!text_mode_rem_bytes) {
        TRACE(("text_getc: eol reached (win = \\r, unix, vms = \\n). cur_state = gs_eol1. rem_bytes=%d, j = %d\n", text_mode_rem_bytes, j));
        text_mode_cur_state = gs_eol1;

#if defined(unix) || defined(__VMS)
        return (int)'\n';
#elif defined(_WIN32) 
        return (int)'\r';
#endif
    }

    window_pos++;
    text_mode_rem_bytes--;
    return (int)*((byte_t*)p_content_window->p_content + blk_pos);
}


//  text mode putc()
int f11_file_sequential::flush_record()
{
    int i, j;

    // Compose proper record - write its cuttent length
    *(ods_word_t*)record_buf = record_len;

    // Record can only have even size
    if (record_len & 1)
	record_len++;

    byte_t*	b = (byte_t*)record_buf;

    for(i = 0; i < record_len + 2; i++ ) {
	j = (this->f11_bin_putc)(*b++);
	if(j == -1) {
	    i = -1;
	    break;
	}
    }
    
    record_len = 0;

    return i;
}

int f11_file_sequential::f11_text_putc(char c)
{
    if (record_len >= MAX_RECORD_LEN) {
	
	// Maximum length of record reached, cannot proceed
	dev->set_error(F11_EOF);
	return -1;
    }

    if(c == '\n')
    {
	if (flush_record() == -1) {
	    return -1;
	}
    } 

#if defined(_WIN32)
    else if (c == '\r')
    {
	// Do nothing
    }
#endif
    else
    {
	record_buf[record_len + 2] = c;
	record_len++;
    }
    return 0;
}
