//
// Copyright (C) 2000 Software Resources International SA
//
// This code is provided on an 'as is' basis.
// Software Resources International SA carries no responsibility for
// the application of this code in any way or form.
//

// Files-11 Structure level 2 

#include <stdlib.h>
#include <string.h>

#include "definitions.h"
#include "f11asrt.h"
#include "devio.h"
#include "files11.h"
#include "bitmap.h"
#include "f11liberr.h"
#include "f11file_sys.h"
#include "f11vol.h"

void	f11_file_indexf_sys::init_bmap()
{
    ods_dword_t*    p_bmap = get_content_bmap();
    bmap.init(p_bmap, dev->hblk.hm2_l_maxfiles);
}

// Allocate first free file index
// This function returns corresponding file number, which is idx + 1
dword_t	f11_file_indexf_sys::alloc_file_index()
{
    dword_t idx = bmap.get_ff_bit();

    if (idx == -1)
    {
        // There is no free file header index: set error and return 
        // index allocated last time 
        dev->set_error(F11_IDX_FULL);
    }
    else
    {
	// Make the current free index busy 
	bmap.alloc_bit(idx);
	TRACE(("INDEXF.SYS: file index %d:%d allocated\n", idx, bmap.get_bit_count()));
    }

    flag_content_modified = 1;

    dword_t fid_num = idx + 1;
    return fid_num;
}

void	f11_file_indexf_sys::release_file_index(dword_t fid_num)
{
    dword_t	idx = fid_num - 1;

    bmap.release_bit(idx);

    TRACE(("INDEXF.SYS: bit %lu for fid_num = %lu released\n", idx, fid_num));

    flag_content_modified = 1;
}

void	f11_file_bitmap_sys::init_bmap()
{
    ODS_SCB*	    p_scb = get_content_scb();
    ods_dword_t*    p_bmap = get_content_bmap();

    bmap.init(p_bmap, p_scb->scb_l_volsize / p_scb->scb_w_cluster);
}

dword_t	f11_file_bitmap_sys::get_free_clusters()
{
    dword_t count = 0;
    dword_t save_ff_bit = bmap.get_ff_bit();

    for (dword_t idx = save_ff_bit; 
	 idx != -1 && idx < bmap.get_bit_count(); 
	 idx++)
    {
	if (bmap.get_bit(idx) == bmap.free_value()) count++;
    }

    return count;
}

// The same as previous but returns the amount of free blocks
dword_t	f11_file_bitmap_sys::get_free_blocks()
{
    return get_free_clusters() * get_content_scb()->scb_w_cluster;
}

// Allocate n contiguous clusters
dword_t	f11_file_bitmap_sys::alloc_clusters(dword_t n)
{
    dword_t save_ff_bit = bmap.get_ff_bit();
    dword_t idx = bmap.get_ff_bit();
    dword_t max_idx = bmap.get_bit_count();

    if(idx == -1)
    {
        // There is no free clusters: set error and return 
        // the index allocated last time 
        dev->set_error(F11_IDX_FULL);
        return idx;
    }

    if (get_free_clusters() < n)
    {
        // There is no free clusters: set error and return 
        // the index allocated last time 
        dev->set_error(F11_IDX_FULL);
        return max_idx + 1;
    }

    dword_t rest = n;
    dword_t found_idx = idx;
    while (++idx < max_idx)
    {
	rest--;
	
	if (rest == 0)
	    break;  // we found n free clusters

	// shift to next bit
	if (idx >= bmap.get_bit_count())
	    break;

	if (bmap.get_bit(idx) != bmap.free_value()) 
	{
	    // Skip to next chunk of free clusters
	    rest = n;
	    found_idx = bmap.update_ff_bit(idx);
	}
    }

    if (rest == 0) {
	// Allocate found clusters!
	for(dword_t i = 0; i < n; i++)
	    bmap.alloc_bit(found_idx + i);
    } else {
	return -1;
    }

    // Update the bitmap starting ff_bit
    bmap.update_ff_bit(save_ff_bit);
    
    flag_content_modified = 1;

    return found_idx;
}

// Allocate n contiguous clusters at fixed position
dword_t	f11_file_bitmap_sys::alloc_clusters_at(dword_t start_at, dword_t n)
{
    dword_t save_ff_bit = bmap.get_ff_bit();
    dword_t idx = start_at;
    dword_t max_idx = bmap.get_bit_count();

    if(idx == -1)
    {
        // There is no free clusters: set error and return 
        // the index allocated last time 
        dev->set_error(F11_IDX_FULL);
        return idx;
    }

    if (get_free_clusters() < n)
    {
        // There is no free clusters: set error and return 
        // the index allocated last time 
        dev->set_error(F11_IDX_FULL);
        return max_idx + 1;
    }

    dword_t rest = n;
    dword_t found_idx = idx;
    do 
    {
	// Shift to next bit
	if (idx >= bmap.get_bit_count())
	    break;
	if (bmap.get_bit(idx) != bmap.free_value()) 
	{
	    break;
	}
	rest--;
	if (rest == 0)
	    break;  // We found n free clusters

    } while (idx++ < max_idx);

    if (rest == 0)
    {
	// Allocate found clusters!
	for(dword_t i = 0; i < n; i++)
	    bmap.alloc_bit(found_idx + i);
    } else {
	return -1;
    }

    // Update the bitmap starting ff_bit
    bmap.update_ff_bit(save_ff_bit);

    flag_content_modified = 1;

    return found_idx;
}

// Release cluster
void f11_file_bitmap_sys::release_cluster(dword_t cluster)
{
    bmap.release_bit(cluster);

    TRACE(("BITMAP.SYS: bit %lu released\n", cluster));

    flag_content_modified = 1;
}
