//
// Copyright (C) 2000 Software Resources International SA
//
// This code is provided on an 'as is' basis.
// Software Resources International SA carries no responsibility for
// the application of this code in any way or form.
//

// Files-11 ODS-2 File Access Library

#ifndef __F11LIB_H__
#define __F11LIB_H__

#if defined(__cplusplus)
extern "C" {
#endif

typedef unsigned char uint8;
typedef unsigned short uint16;
typedef unsigned int uint32;

struct DateType {
    uint8	Century;
    uint8	Year;
    uint8	Month;
    uint8	Day;
    uint8	Hour;
    uint8	Minute;
    uint8	Second;
    uint8	CentSec;
};

struct UICType {
    uint8	Member;
    uint8	Group;
};

struct ProtType {
    uint8	System;
    uint8	Owner;
    uint8	Group;
    uint8	World;
};

struct RMSType {
    uint8	RecType;
    uint8	FilOrg;
    uint8	RecAttrib;
    uint32	RecSize;
    uint32	EOFBlk;
    uint32      FFByte;
    uint32	MaxRec;
};

struct VolInfoType {
    uint32	volsize;
    uint32	sectors;
    uint32	cylinder;
    uint32	tracks;
    uint16	rvn;
    uint16	cluster;
    uint32	maxfiles;
    uint16	extend;
    char	ownername[12];
    uint32	freeblocks;

    char	volname[12];
    struct UICType	volowner;
    struct ProtType	fileprot;
};

struct FileInfoType
{
    char	Name[87];	    // as string
    struct DateType	CreDate;    // contains file creation date
    struct DateType	RevDate;    // contains date of the last revision
    struct UICType	UIC;
    struct ProtType	Prot;	    // file protection code
    struct RMSType	RMSAttr;
    uint8	FileChar[4];
    uint16	StrucLev;	    // structure level
    uint32	FileNum;	    // file index number
    uint16	SeqNum;		    // file sequence number
    uint16	RVN;		    // file rvn
    uint32	UsedBlocks;
    uint32	AllocBlocks;
    uint32	WCC;		    // context
};

#include "f11lib_share.h"

gblfunc_f11lib int F11_Install(void);
gblfunc_f11lib int F11_DeInstall(void);
gblfunc_f11lib int F11_Assign(const char* device_name, int read_only);
gblfunc_f11lib int F11_Deassign(int device_handle);
gblfunc_f11lib int F11_GetVolumeInfo(struct VolInfoType* volume_info);
gblfunc_f11lib int F11_ReadDir(const char* file_spec, struct FileInfoType* dir_info);
gblfunc_f11lib int F11_OpenFile(const char* file_spec, int text_mode);
gblfunc_f11lib int F11_GetFileInfo(struct FileInfoType* fi);
gblfunc_f11lib int F11_Read(void* buffer, int bytes);
gblfunc_f11lib int F11_Close(void);
gblfunc_f11lib int F11_DirDown(const char* dir_spec);
gblfunc_f11lib int F11_DirUp(void);
gblfunc_f11lib int F11_CreateFile(const char* file_spec, int text_mode, int fsize);
gblfunc_f11lib int F11_Write(void* buffer, int bytes);
gblfunc_f11lib int F11_MkDir(const char* file_spec);
gblfunc_f11lib int F11_Delete(const char* file_spec);

gblfunc_f11lib int F11_GetError(void);		// returns library system indep error codes
gblfunc_f11lib void F11_SetError(int err);	// sets library error
gblfunc_f11lib int F11_GetDevError(void);	// returns system dep device error if DRIVE_ERR
gblfunc_f11lib int F11_SetTraceOn(void);	// print out trace info
gblfunc_f11lib int F11_SetTraceOff(void);	// do not print trace info

#if defined(__cplusplus)
}
#endif

#endif
