//
// Copyright (C) 2000 Software Resources International SA
//
// This code is provided on an 'as is' basis.
// Software Resources International SA carries no responsibility for
// the application of this code in any way or form.
//

// Win32 implementation

#include <windows.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "definitions.h"
#include "f11asrt.h"
#include "files11.h"
#include "devio.h"
#include "fileio.h"

HANDLE fileio_open_file(const char* fn, int mode)
{
    HANDLE handle;
    handle = CreateFile(
	fn,
	((mode & FILEIO_RO) ? 0 : GENERIC_WRITE) | GENERIC_READ,
	0,
	NULL,
	((mode & FILEIO_RO) ? OPEN_EXISTING : CREATE_ALWAYS),
	FILE_FLAG_RANDOM_ACCESS,
	0);

    return handle;
}

int    fileio_read_file(HANDLE h, void* buf, int length)
{
    DWORD   r_len;
    return ReadFile(h, buf, length, &r_len, NULL) ? r_len : -1;    
}

int    fileio_write_file(HANDLE h, void* buf, int length)
{
    DWORD   r_len;

    return WriteFile(h, buf, length, &r_len, NULL) ? r_len : -1;    
}

void   fileio_close_file(HANDLE h)
{
    CloseHandle(h);
}

dword_t fileio_file_size(const char* fs)
{
    struct _stat st;

    if (_stat(fs, &st) != 0) 
	return (dword_t)-1;
    return st.st_size;
}

int fileio_file_chdir(const char* dir)
{
    return SetCurrentDirectory(dir) ? 0 : -1;
}

char* fileio_file_getwd(char* dir, int size)
{
    ASSERT(dir && size);
    if (!GetCurrentDirectory(size, dir)) dir[0] = '\0';
    return dir;
}

char* fileio_absolute_file_name(const char* rel, char* absolute, int len)
{
    LPTSTR  fn;

    if (GetFullPathName(rel, len, absolute, &fn) == 0) {
	strncpy(absolute, rel, len);
    }
    return absolute;
}
